Attribute VB_Name = "Battlenet"
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''' Battle.net String Parser                           '''
'''                                                    '''
''' by Michael Horbatsch (Maglor)                      '''
''' Email: maglor@alphanine.com                        '''
''' ICQ: 26380980                                      '''
''' URL: http://doa.dnserve.net                        '''
'''                                                    '''
''' This module contains functions for connecting to   '''
''' Battle.net, Blizzard Entertainment's free chat and '''
''' gaming service.                                    '''
'''                                                    '''
''' Requires: Visual Basic 5.0+                        '''
'''           Microsoft Winsock Control                '''
'''                                                    '''
''' You may use this code free of charge, without any  '''
''' permission. You may distribute it freely, without  '''
''' modifying it. You may use this code freely in your '''
''' programs, however, if you release a program to the '''
''' public, please give credit to Michael Horbatsch    '''
''' (Maglor) for this code.                            '''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit
DefInt I
DefStr S
DefLng L

Public Sub Connect(ctlSock As Control, strServer, Optional strPort = 6112)
    'Connect to a Battle.net Server
    
    'ctlSock  : A Microsoft Winsock Control v5.0+
    'strServer: The Battle.net server
    'strPort  : The remote port (don't change this unless you know what you're doing)
    
    If ctlSock.State <> 0 Then ctlSock.Disconnect
    With ctlSock
        .RemoteHost = strServer
        .RemotePort = strPort
        .Protocol = sckTCPProtocol
        .Connect
    End With
End Sub

Public Sub Login(ctlSock As Control, strUser, strPass)
    'Login to the Battle.net Server. This should be called once a
    'connection has been established. The event Winsock_Connect
    'indicates that a connection has been established.
    
    'ctlSock    : The Winsock Control which established the connection
    'strUser    : Your username (use GUEST or ANONYMOUS if you don't have one)
    'strPassword: Your password (GUEST and ANONYMOUS don't require passwords)

    If ctlSock.State <> 0 Then
        ctlSock.SendData Chr(3) & Chr(4) & strUser & vbCrLf & strPass & vbCrLf
    End If
End Sub

Public Sub Disconnect(ctlSock As Control)
    'Disconnect from Battle.net
    
    'ctlSock: The Winsock Control
    ctlSock.Close
End Sub

Public Sub ParseData(strData)
    'Parse a data string which has been received from Battle.net.
    'This should be called whenever a Winsock_DataArrival event
    'occurs, where strData is the received data string.
    
    Dim strStrings() As String, i
    
    'Sometimes Battle.net puts several strings into one line,
    'separated by CrLf's. Here we split and parse them.
    
    SplitStr strData, strStrings, vbCrLf
    For i = 1 To UBound(strStrings)
        Parse2 strStrings(i)
    Next i
End Sub

Private Sub Parse2(ByVal strData As String)
    'This subroutine does the actual parsing. It gets called
    'by the ParseData subroutine. There is no need to call it
    'from outside.
    
    Dim strData1, strData2, i
    Dim strArgs() As String
    
    'Sometimes, Battle.net puts a LineFeed character at the
    'beginning of strings. We need to trim it.
    If Left(strData, 1) = Chr(10) Then
        strData = Right(strData, Len(strData) - 1)
    End If
    
    'Check whether this is normal text, or an event
    If Val(Left(strData, 4)) < 1001 Or Val(Left(strData, 4)) > 3000 Then
        Event_Unknown strData
        Exit Sub
    End If
    
    'If this is an event, then we separate the arguments, and quoted text
    i = InStr(1, strData, Chr(34), vbBinaryCompare)
    If i <> 0 Then
        strData1 = Left(strData, i - 2)
        strData2 = Mid(strData, i + 1, Len(strData) - i - 1)
    Else
        strData1 = strData
        strData2 = ""
    End If
    SplitStr strData1, strArgs(), " "
    
    'Call Appropriate Event based on ID
    Select Case Val(strArgs(1))
        Case 1001: Event_User strArgs(3), strArgs(4), strArgs(5)
        Case 1002: Event_Join strArgs(3), strArgs(4), strArgs(5)
        Case 1003: Event_Leave strArgs(3), strArgs(4)
        Case 1004: Event_RecvWhisper strArgs(3), strArgs(4), strData2
        Case 1005: Event_Talk strArgs(3), strArgs(4), strData2
        Case 1006: Event_Broadcast strData2
        Case 1007: Event_Channel strData2
        Case 1009: Event_Flags strArgs(3), strArgs(4)
        Case 1010: Event_SendWhisper strArgs(3), strArgs(4), strData2
        Case 1013: Event_ChannelFull strData2
        Case 1014: Event_ChannelNotExist strData2
        Case 1015: Event_ChannelRestricted strData2
        Case 1016: Event_Info strData2
        Case 1018: Event_Info strData2
        Case 1019: Event_Error strData2
        Case 1023: Event_Emote strArgs(3), strArgs(4), strData2
        Case 2010: Event_Name strArgs(3)
        Case 3000: Event_Info strData2
    End Select
End Sub

Public Function SplitStr(ByVal OriginalString As String, ByRef ReturnArray() As String, ByVal Delimiter As String) As Long
    'Used to split a string by delimiters, into a dynamic array, and returns
    'the ubound of the array
    
    Dim sItem, lArrCnt
    Dim lLen, lPos
    
    lArrCnt = 0
    lLen = Len(OriginalString)

    Do
        lPos = InStr(1, OriginalString, Delimiter, vbTextCompare)
        If lPos <> 0 Then
            sItem = Left$(OriginalString, lPos - 1)
            OriginalString = Mid$(OriginalString, lPos + 1)
            If sItem <> "" Then
                lArrCnt = lArrCnt + 1
                If lArrCnt = 1 Then
                    ReDim ReturnArray(1 To lArrCnt) As String
                Else
                    ReDim Preserve ReturnArray(1 To lArrCnt) As String
                End If
                ReturnArray(lArrCnt) = sItem
            End If
        End If
    Loop While lPos <> 0

    If OriginalString <> "" Then
        lArrCnt = lArrCnt + 1
        ReDim Preserve ReturnArray(1 To lArrCnt) As String
        ReturnArray(lArrCnt) = OriginalString
    End If
    
    SplitStr = lArrCnt
End Function

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'''''   BELOW ARE THE EVENTS CALLED BY THE DATA PARSER   '''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Sub Event_Unknown(strText)
    'Normal text from the server (not an event)
    'strText: Text Message
    
    frmBot.ShowTxt strText & vbCrLf
End Sub

Private Sub Event_User(strUser, strFlag, strProduct)
    'Listing of a user in the current channel (after a Channel event)
    'strUser   : User's Name
    'strFlag   : User's Status Flag
    'strProduct: User's Product
    
    frmBot.ShowTxt strUser & " is here using " & strProduct & vbCrLf
End Sub

Private Sub Event_Join(strUser, strFlag, strProduct)
    'User joins current channel
    'strUser   : User's Name
    'strFlag   : User's Status Flag
    'strProduct: User's Product
    
    frmBot.ShowTxt strUser & " joined using " & strProduct & vbCrLf
End Sub

Private Sub Event_Leave(strUser, strFlag)
    'User leaves current channel
    'strUser: User's Name
    'strFlag: User's Status Flag
    
    frmBot.ShowTxt strUser & " left" & vbCrLf
End Sub

Private Sub Event_RecvWhisper(strUser, strFlag, strText)
    'Received Whisper
    'strUser: User's Name
    'strFlag: User's Status Flag
    'strText: Whisper Text
    
    frmBot.ShowTxt "<From: " & strUser & "> " & strText & vbCrLf
End Sub

Private Sub Event_Talk(strUser, strFlag, strText)
    'User Talks
    'strUser: User's Name
    'strFlag: User's Status Flag
    'strText: Whisper Text
    
    frmBot.ShowTxt "<" & strUser & "> " & strText & vbCrLf
End Sub

Private Sub Event_Broadcast(strText)
    'Broadcast from server administrator
    'strText: Broadcast Message
    
    frmBot.ShowTxt "Broadcast Message: " & strText & vbCrLf
End Sub

Private Sub Event_Channel(strText)
    'Joined New Channel
    'strText: Channel Name
    
    frmBot.ShowTxt "Joined Channel: " & strText & vbCrLf
End Sub

Private Sub Event_Flags(strUser, strFlag)
    'User Changes Status Flags
    'strUser: User's Name
    'strFlag: User's Status Flag
End Sub

Private Sub Event_SendWhisper(strUser, strFlag, strText)
    'You Send a Whisper
    'strUser: User's Name
    'strFlag: User's Status Flag
    'strText: Whisper Text
    
    frmBot.ShowTxt "<To: " & strUser & "> " & strText & vbCrLf
End Sub

Private Sub Event_ChannelFull(strText)
    'Channel is Full
    'strText: Name of Channel
    
    frmBot.ShowTxt "Chanenl " & strText & " is full" & vbCrLf
End Sub

Private Sub Event_ChannelNotExist(strText)
    'Channel does not Exist
    'strText: Name of Channel
    
    frmBot.ShowTxt "Channel " & strText & " does not exist" & vbCrLf
End Sub

Private Sub Event_ChannelRestricted(strText)
    'Channel is Restricted
    'strText: Name of Channel
    
    frmBot.ShowTxt "Channel " & strText & " is restricted" & vbCrLf
End Sub

Private Sub Event_Info(strText)
    'Information from the Server
    'strText: Info Message
    
    frmBot.ShowTxt strText & vbCrLf
End Sub

Private Sub Event_Error(strText)
    'Error from the Server
    'strText: Error Message
    
    frmBot.ShowTxt strText & vbCrLf
End Sub

Private Sub Event_Emote(strUser, strFlag, strText)
    'Emote Event
    'strUser: User's Name
    'strFlag: User's Status Flag
    
    frmBot.ShowTxt "<" & strUser & " " & strText & ">" & vbCrLf
End Sub

Private Sub Event_Name(strUser)
    'Username Assignment
    'strUser: Name which Battle.net assigns to you (once during login)
    
    frmBot.ShowTxt "Assigned Name: " & strUser & vbCrLf
    frmBot.strName = strUser
    MsgBox frmBot.strName
End Sub
